/*
 * Copyright © 2016 - 2017 Copyright (c) Sprint, Inc. and others.  All rights reserved.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 */
package org.onosproject.fpcagent.util.eventStream;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * SSE Server implementation
 */
public class JettyServer implements AutoCloseable {
    private static final Logger log = LoggerFactory.getLogger(JettyServer.class);
    private static JettyServer _instance;
    private Server server;

    public static JettyServer createInstance() {
        if (_instance == null) {
            _instance = new JettyServer();
        }
        return _instance;
    }

    public static JettyServer getInstance() {
        return _instance;
    }

    /**
     * Method used to initialize and start the SSE server
     */
    public void open() {
        ExecutorService executorService = Executors.newSingleThreadExecutor();
        executorService.submit(() -> {
            try {
                server = new Server(8070);
                ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
                context.setContextPath("/");

                context.addServlet(new ServletHolder(ResponseServer.class), "/response");
                context.addServlet(new ServletHolder(NotificationServer.class), "/notification");

                server.setHandler(context);

                context.addEventListener(new ConfigureService());
                context.addEventListener(new NotificationService());

                try {
                    server.start();
                    log.info("Jetty-Server Started");
                    server.join();
                } catch (Exception e) {
                    log.error(ExceptionUtils.getFullStackTrace(e));
                } finally {
                    server.destroy();
                }
            } catch (Exception e) {
                log.error(ExceptionUtils.getFullStackTrace(e));
            }
        });
    }

    @Override
    public void close() {
        try {
            server.stop();
        } catch (Exception e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        }
    }
}

